/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.extension.data;

import java.util.Comparator;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.base.ConstraintViolationBase;

/**
 * This interface describes a constraint violation produced by some constraint
 * checker. A constraint violation has a source object, a severity of type
 * {@link ESeverity}, an explanation of type String and possibly a quick fix
 * handler of type {@link IQuickFixHandler}.
 * <p>
 * Sub-classes should adapt to {@link EObject} in {@link #getAdapter(Class)} and
 * return the source element.
 * 
 * @see ConstraintViolationBase
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 7C5D15AAF9D6703ABFECFDF82B5D45B8
 */
public interface IConstraintViolation<T extends EObject> extends IAdaptable {

	/** Returns the source of the constraint violation. */
	T getSource();

	/** Returns the severity of the constraint violation. */
	ESeverity getSeverity();

	/** Returns the explanation of the constraint violation. */
	String getExplanation();

	/** Returns the quick fix handler for this constraint violation. */
	IQuickFixHandler getQuickFixHandler();

	/** Enumeration of severity classes. */
	public static enum ESeverity {
		/** Fatal error severity. */
		FATAL,
		/** Normal error severity. */
		ERROR,
		/** Warning severity. */
		WARNING,
		/** Information severity. */
		INFO,
		/** Debug severity. */
		DEBUG,

		/** The lowest None severity. */
		NONE;

		/** Translates ESeverity to int. */
		public static int getIntSeverity(ESeverity severity) {
			switch (severity) {
			case FATAL:
				return -5;
			case ERROR:
				return -3;
			case WARNING:
				return -1;
			case INFO:
				return 1;
			case DEBUG:
				return 5;
			}
			// ESeverity.NONE
			return 10000;
		}

		/** Returns the lowest severity level. */
		public static ESeverity lowest() {
			return NONE;
		}
	}

	/** Interface for quick fix handlers. */
	public static interface IQuickFixHandler {
		/** Applies the quick fix. */
		void applyQuickFix();
	}

	/** Stores the severity comparator used during violation sorting. */
	public static final Comparator<IConstraintViolation<? extends EObject>> SEVERITY_COMPARATOR = new Comparator<IConstraintViolation<? extends EObject>>() {
		/** {@inheritDoc} */
		@Override
		public int compare(IConstraintViolation<? extends EObject> arg0,
				IConstraintViolation<? extends EObject> arg1) {
			int sev0 = ESeverity.getIntSeverity(arg0.getSeverity());
			int sev1 = ESeverity.getIntSeverity(arg1.getSeverity());
			return sev0 - sev1;
		}
	};
}
